import { Component, OnInit, ElementRef, ViewChild, AfterViewInit, Input } from '@angular/core';
import * as PIXI from 'pixi.js';
import { EventEmitter } from 'events';
import { EventManager } from '@angular/platform-browser';
import { OutlineFilter, OldFilmFilter  } from 'pixi-filters';
import { CanvasShareDataService, GameMode } from '../canvas-share-data.service';
import {CacheTokenService} from '../http-interceptors/cache-token.service'; // 引入自动登录 获取token服务
import * as ObjectID from 'bson-objectid';
import { Router } from '@angular/router';
import { isThisSecond } from 'date-fns';
import { Charm } from './charm';


@Component({
  selector: 'app-working-area',
  templateUrl: './working-area.component.html',
  styleUrls: ['./working-area.component.scss']
})

export class WorkingAreaComponent extends EventEmitter implements OnInit, AfterViewInit {

  constructor(private eventManager: EventManager, public canvasData: CanvasShareDataService, public token: CacheTokenService, private router: Router, ) {
    super();
  }

  @ViewChild('content')
  content: ElementRef;
  /**
   * 父组件
   */
  @Input() init: any;
  /**
   * pixijs 程序
   */
  public app: PIXI.Application;
  /**
   * 资源加载器
   */
  public loader = PIXI.Loader.shared;
  /**
   * 背景图
   */
  public backgroundImage: PIXI.Sprite;
  /**
   * 预览单点图标
   */
  public previewSinglePointIcon = new PIXI.Sprite();
  /**
   * 预览线段
   */
  public previewLineSegment = new PIXI.Graphics();
  /**
   * 预览原点
   */
  public circleShadow = new PIXI.Graphics();
  /**
   * 鼠标位置
   */
  public mousePosition: PIXI.Point = new PIXI.Point(0, 0);
  /**
   * 绘画模式
   */
  private paintMode: PaintMode;
  /**
   * 选择器
   */
  public selection: Selection = new Selection(this);
  /**
   * 当前鼠标的点
   */
  public currentClickPoint: PIXI.Graphics = new PIXI.Graphics();
  /**
   * 绘制点集合
   */
  public paintPoints: PIXI.Point[] = [];
  /**
   * 绘制中的管线
   */
  public paintingPipeline: Pipeline = null;
  /**
   * 绘制中的箭头
   */
  public paintingArrows: Arrows = null;
  /**
   * 绘制中的多点图标
   */
  public paintingIcon: MultipointIcon;
  /**
   * 绘制中的连线
   */
  public paintingLine: PIXI.Graphics = new PIXI.Graphics();
  /**
   * 绿色描边
   */
  public outlineFilterGreen = new OutlineFilter(2, 0x00ff00);
  /**
   * 拷贝素材数据
   */
  public copyData: any[] = [];
  /**
   * 确认绘制按钮
   */
  private enterPaintEndButton = PIXI.Sprite.from('assets/images/enterPaintButton.png');
  /**
   * 框选工具图形
   */
  private rectToolGraphics = new PIXI.Graphics();
  /**
   * 初始鼠标位置
   */
  private initialScreenMousePos: PIXI.Point = new PIXI.Point();
  /**
   * 最终鼠标位置
   */
  private finalScreenMousePos: PIXI.Point = new PIXI.Point();
  /**
   * 允许编辑
   */
  public allowEdit = false;

  public c;
  public animation;
  public animationIcon;
  public animationTime;
  // 车辆作业面
  public carAreas: PolygonIcon[];
  // 车辆数据
  public carData: Map<string, any> = new Map<string, any>();
  // 当前选择的车辆id
  public selectCar: any = null;
  // 根据ID 找到数据

  // 是否登录
  public verificationURL = '/ui/plan'; // 验证url地址是否为验证页面
  public verificationURLTwo = '/ui/similarPlans'; // 验证url地址是否为验证页面
  count = 0;
  isLogin() {
    const token = sessionStorage.getItem('token'); // 判断 是否登录状态
    if (token) {
      // this.init.getAllLibrary() //获取素材库
      this.init.getAllBuildings(); // 获取所有建筑
      this.init.getAllFirePlan(); // 获取所有灾情
      this.init.getSitePlan(); // 调用父组件 初始化方法
    } else {
      this.token.login().then(res => {
        this.init.params.companyId = sessionStorage.getItem('companyId');
        // this.init.getAllLibrary() //获取素材库
        this.init.getAllBuildings(); // 获取所有建筑
        this.init.getAllFirePlan(); // 获取所有灾情
        this.init.getSitePlan(); // 调用父组件 初始化方法
      });
    }
  }
  // 是否登录

  ngOnInit(): void {
    this.eventManager.addGlobalEventListener('window', 'keydown', (event: any) => {
      if (event.keyCode === 17) {
        this.selection.isMultiselection = true;
      }
    });
    this.eventManager.addGlobalEventListener('window', 'keyup', (event: any) => {
      if (event.keyCode === 17) {
        this.selection.isMultiselection = false;
        this.rectToolGraphics.visible = false;
        this.rectToolGraphics.clear();
      }
      if (event.keyCode === 46) {
        this.selection.objects.forEach(item => {
          // 删除 选中的数据
          if (item.assetData?.IsFromBuilding) {
            // console.log(this.canvasData.originaleveryStoreyData.data[item.assetData.Id]);
            delete this.canvasData.originalcompanyBuildingData?.data[item.assetData.Id];
            // console.log(this.canvasData.originalcompanyBuildingData.data[item.assetData.Id]);
          } else {
            // console.log(this.canvasData.originaleveryStoreyData.data[item.assetData.Id]);
            delete this.canvasData.originaleveryStoreyData?.data[item.assetData.Id];
            // console.log(this.canvasData.originaleveryStoreyData.data[item.assetData.Id]);
          }
          // console.log(this.backgroundImage.getChildByName(item.assetData.Id));
          // 删除选中的图标
          this.backgroundImage.removeChild(this.backgroundImage.getChildByName(item.assetData.Id));
          // 标记
          this.canvasData.isChange = true;
        });
        this.emit('deleteIcon');
      }
    });
    // 测试代码
    this.eventManager.addGlobalEventListener('window', 'keypress', (event: any) => {

    });
    // setTimeout(() => {
    //   this.createCanvas();
    // }, 0);
  }

  ngAfterViewInit(): void {
    this.createCanvas();
    window.onresize = () => {
      this.resizeCanvas();
    };
  }
  /**
   *
   * @param event 鼠标滑动事件
   */
  public mouseWheelHandel(event) {
    const delX = this.mousePosition.x - this.backgroundImage.position.x;
    const delY = this.mousePosition.y - this.backgroundImage.position.y;
    const pivot = this.backgroundImage.toLocal(this.mousePosition);
    const delta = Math.max(-1, Math.min(1, (event.wheelDelta || -event.detail)));
    if (delta > 0) {
      if (this.backgroundImage.scale.x >= 32) {
        this.backgroundImage.scale.x = 32;
        this.backgroundImage.scale.y = 32;
        this.emit('backgroundScale', this.backgroundImage.scale.x);
        return;
      }
      this.backgroundImage.pivot.set(pivot.x, pivot.y);

      this.backgroundImage.scale.x += this.backgroundImage.scale.x * 0.1;
      this.backgroundImage.scale.y += this.backgroundImage.scale.y * 0.1;

      this.backgroundImage.position.x += delX;
      this.backgroundImage.position.y += delY;
    } else if (delta < 0) {
      if (this.backgroundImage.scale.x <= 0.1) {
        this.backgroundImage.scale.x = 0.1;
        this.backgroundImage.scale.y = 0.1;
        this.emit('backgroundScale', this.backgroundImage.scale.x);
        return;
      }
      this.backgroundImage.pivot.set(pivot.x, pivot.y);

      this.backgroundImage.scale.x -= this.backgroundImage.scale.x * 0.1;
      this.backgroundImage.scale.y -= this.backgroundImage.scale.y * 0.1;

      this.backgroundImage.position.x += delX;
      this.backgroundImage.position.y += delY;
    }
    this.emit('backgroundScale', this.backgroundImage.scale.x);
  }
  /**
   *
   * @param icon 移动到选中车辆到屏幕中心点
   */
  public moveIconToScreenCenter(icon) {
    if (icon.parent === this.backgroundImage && (
      icon.assetData.Type === 1 ||
      icon.assetData.Type === 2 ||
      icon.assetData.Type === 3 ||
      icon.assetData.Type === 4
    )) {
      console.log(this.backgroundImage.position);
      this.backgroundImage.pivot.set(icon.x, icon.y);
      this.backgroundImage.position.set(771, 404);
      clearTimeout(this.animationTime);
      this.animation?.pause();
      this.animationIcon?.scale.set(1);
      this.animation = this.c.breathe(icon, 10, 10, 30, true, 0);
      this.animationIcon = icon;
      this.animationTime = setTimeout(() => {
        this.animation?.pause();
        this.animationIcon?.scale.set(1);
      }, 5000);
    }
  }
  /**
   * 创建画布
   */
  private createCanvas(): void {
    this.app = new PIXI.Application({
      width: this.content.nativeElement.clientWidth,
      height: this.content.nativeElement.clientHeight,
      antialias: true,
      transparent: false,
      resolution: 1,
      backgroundColor: 0xE9FAFF
    });
    this.content.nativeElement.appendChild(this.app.view);
    const url = this.router.url;
    // tslint:disable-next-line: no-unused-expression
    url === this.verificationURL || url === this.verificationURLTwo ? this.isLogin() : null;
    this.c = new Charm(PIXI);
    this.createBackgroundImage();
    // this.createPreviewSinglePointIcon();
    // this.createPreviewLineSegment();
    // this.createCircleShadow();
    // this.createEnterPaintEndButton();
    // this.backgroundImage.addChild(this.paintingLine);
    this.on('select', obj => {
      console.log(obj);
      this.moveIconToScreenCenter(obj);
      if (this.allowEdit) {
        if (obj instanceof MultipointIcon) {
          obj.setPointVisiable(true);
        } else if (obj instanceof PolygonIcon) {
          obj.setPointVisiable(true);
        } else {
          obj.filters = [this.outlineFilterGreen];
        }
      } else {
        obj.filters = [this.outlineFilterGreen];
      }
    });
    this.on('deselect', obj => {
      if (this.allowEdit) {
        if (obj instanceof MultipointIcon) {
          obj.setPointVisiable(false);
        } else if (obj instanceof PolygonIcon) {
          obj.setPointVisiable(false);
        } else {
          obj.filters = [];
        }
      } else {
        obj.filters = [];
      }
    });
    this.on('backgroundScale', scale => {
      this.previewSinglePointIcon.scale.set(1 / this.backgroundImage.scale.x);
      this.backgroundImage.children.forEach(item => {
        if (item instanceof SinglePointIcon) {
          if (item.assetData.FixedSize) {
            const data = 1 / scale;
            item.scale.set(data);
          } else {
            const data = 1 / scale;
            item.text.scale.set(data);
          }
        } else if (item instanceof MultipointIcon) {
          const data = 1 / scale;
          item.text.scale.set(data);
        } else if (item instanceof PolygonIcon) {
          const data = 1 / scale;
          item.text.scale.set(data);
        }
      });
    });

    this.app.ticker.add((delta) => {
      this.c.update();

      this.mousePosition = this.app.renderer.plugins.interaction.mouse.global;

      this.previewSinglePointIcon.position = this.backgroundImage.toLocal(this.mousePosition);
      this.circleShadow.position = this.backgroundImage.toLocal(this.mousePosition);

      this.refreshPreviewLineSegment(this.currentClickPoint.position, this.circleShadow.position);

      if (this.rectToolGraphics.visible === true) {

        const init = this.initialScreenMousePos;
        const final = this.finalScreenMousePos;

        this.rectToolGraphics.clear();
        this.rectToolGraphics.lineStyle(2, 0x00ff00, 1);
        this.rectToolGraphics.beginFill(0xccccf2, 0.25);
        this.rectToolGraphics.drawRect(init.x, init.y, final.x - init.x, final.y - init.y);
        this.rectToolGraphics.endFill();
        this.rectToolGraphics.closePath();
      }
      if (this.paintingArrows !== null) {
        this.paintingArrows.assetData.pointB = new PIXI.Point(this.circleShadow.position.x, this.circleShadow.position.y);
        this.paintingArrows.refresh();
      }
    });
  }
  public resizeCanvas() {
    this.app.renderer.resize(this.content.nativeElement.clientWidth, this.content.nativeElement.clientHeight);
  }
  /**
   * 设置名称显示
   * @param value true  显示 false 隐藏
   * @param mode BasicInformation = 0 基本信息 Assignment想定作业 = 1 想定作业
   */
  public setNameVisible(value: boolean, mode: GameMode): void {
    this.backgroundImage.children.forEach(item => {
      if (item instanceof SinglePointIcon) {
        item.setNameVisible(value, mode);
      } else if (item instanceof MultipointIcon) {
        item.setNameVisible(value, mode);
      } else if (item instanceof PolygonIcon) {
        item.setNameVisible(value, mode);
      }
    });
  }
  /**
   * 根据id刷新图标
   * @param id 图标数据id
   */
  public refreshIcon(id: string): void {
    const icon = this.backgroundImage.children.find(item => item.name === id);
    if (icon instanceof SinglePointIcon) {
      icon.refresh();
    } else if (icon instanceof MultipointIcon) {
      icon.refresh();
    } else if (icon instanceof PolygonIcon) {
      icon.refresh();
    }
  }
  /**
   *
   * @param value 缩放倍数
   */
  public setIconScale(value: number): void {
    this.backgroundImage.children.forEach(item => {
      if (item instanceof SinglePointIcon) {
        item.scale.set(value);
      } else if (item instanceof MultipointIcon) {

      } else if (item instanceof PolygonIcon) {

      }
    });
  }
  /**
   * 设置高亮
   */
  public setHighlight(ids: string[]): void {
    this.selection.deselectAll();
    ids.forEach(item => {
      let obj = this.backgroundImage.getChildByName(item);
      if (obj === null) {
        obj = this.app.stage.getChildByName(item);
      }
      this.selection.select(obj);
    });
  }
  /**
   * 刷新
   */
  public refresh(): void {
    this.destroyBackgroundImage();
    this.createBackgroundImage();
    this.refreshBackgroundImage();
    this.versionChecking();


    const floorData = this.canvasData.originaleveryStoreyData.data;
    const buildingData = this.canvasData.originalcompanyBuildingData.data;
    const floor = this.canvasData.selectStorey;
    // console.log(JSON.stringify(buildingData));
    // // key=>属性名    data[key]=>属性值
    Object.keys(floorData).forEach((key) => {
      switch (floorData[key].InteractiveMode) {
          case 0:
          const singleIcon = new SinglePointIcon(floorData[key], this);
          break;
          case 1:
          const icon = new MultipointIcon(floorData[key], this);
          break;
          case 2:
          const polygonIcon = new PolygonIcon(floorData[key], this);
          break;
        }
    });
    Object.keys(buildingData).forEach((key) => {
      if (buildingData[key].FloorId === floor.id) {
        switch (buildingData[key].InteractiveMode) {
          case 0:
            const singleIcon = new SinglePointIcon(buildingData[key], this);
            break;
          case 1:
            const icon = new MultipointIcon(buildingData[key], this);
            break;
          case 2:
            const polygonIcon = new PolygonIcon(buildingData[key], this);
            break;
        }
      }
    });

    const nodeData = this.canvasData.selectPanelPoint?.data;
    if (nodeData !== undefined) {
      Object.keys(nodeData).forEach((key) => {
        // console.log(nodeData[key]);
        Object.keys(nodeData[key]).forEach((tempKey) => {
          console.log(nodeData[key][tempKey]);
          switch (nodeData[key][tempKey].InteractiveMode) {
          case 0:
              const singleIcon = new SinglePointIcon(nodeData[key][tempKey], this);
              break;
          case 1:
              const icon = new MultipointIcon(nodeData[key][tempKey], this);
              break;
          case 2:
              const polygonIcon = new PolygonIcon(nodeData[key][tempKey], this);
              break;
        }
        });
      });
      console.log(this.canvasData.selectPanelPoint);
    }
  }
  /**
   * 加载测试界面
   */
  public loadDemoScene(list: any[], backgroundScale: number) {
    this.backgroundImage.texture = PIXI.Texture.from('./assets/images/demo.jpg');
    this.setBackgroundScale(backgroundScale);
    list.forEach(element => {
      const single = new SinglePointIcon(element, this);
    });

    const putCarAreaData = [
      {
      Id: '5f6719a4e03abe3856d87d7b',
      Name: '作业面1',
      Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 90,
      Color: '#00ff0080',
      Point: {
        x: 0,
        y: 0
      },
      Width: 32,
      Border: {
        w: 0,
        x: 0,
        y: 0,
        z: 0
      },
      Height: 32,
      Enabled: true,
      FloorId: '5f670bb79e91f4410005f564',
      DrawMode: 0,
      FillMode: 1,
      GameMode: 0,
      ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
      CompanyId: '5f67042b9e91f4410005f55c',
      FixedSize: false,
      CanConnection: true,
      ConnectionObjs: [],
      Thickness: 0,
      BuildingId: '5f6704859e91f4410005f55f',
      MultiPoint: [
        {
          x: 2139,
          y: 399
        },
        {
          x: 2194,
          y: 399
        },
        {
          x: 2194,
          y: 1217
        },
        {
          x: 2139,
          y: 1217
        },
      ],
      TemplateId: '5e7b38610137e12020bd7f7b',
      FireElementId: '5e7967f30814180f1c1ecaab',
      PropertyInfos: [],
      IsFromBuilding: true,
      InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面2',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 180,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 5148 - 4000,
            y: 3133 - 2830
          },
          {
            x: 6093 - 4000,
            y: 3133 - 2830
          },
          {
            x: 6093 - 4000,
            y: 3188 - 2830
          },
          {
            x: 5148 - 4000,
            y: 3188 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面3',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 270,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 5061 - 4000,
            y: 3263 - 2830
          },
          {
            x: 5116 - 4000,
            y: 3263 - 2830
          },
          {
            x: 5116 - 4000,
            y: 4047 - 2830
          },
          {
            x: 5061 - 4000,
            y: 4047 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面4',
        Angle: 0,
        Type: [2, 3, 4],
        Direction: 180,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 5149 - 4000,
            y: 4097 - 2830
          },
          {
            x: 6094 - 4000,
            y: 4097 - 2830
          },
          {
            x: 6094 - 4000,
            y: 4152 - 2830
          },
          {
            x: 5149 - 4000,
            y: 4152 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面5',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 180,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 6191 - 4000,
            y: 3133 - 2830
          },
          {
            x: 6942 - 4000,
            y: 3133 - 2830
          },
          {
            x: 6942 - 4000,
            y: 3188 - 2830
          },
          {
            x: 6191 - 4000,
            y: 3188 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面6',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 90,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 6139 - 4000,
            y: 2378 - 2830
          },
          {
            x: 6194 - 4000,
            y: 2378 - 2830
          },
          {
            x: 6194 - 4000,
            y: 3095 - 2830
          },
          {
            x: 6139 - 4000,
            y: 3095 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面7',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 90,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 7060  - 4000,
            y: 2340 - 2830
          },
          {
            x: 7115  - 4000,
            y: 2340 - 2830
          },
          {
            x: 7115  - 4000,
            y: 3057 - 2830
          },
          {
            x: 7060  - 4000,
            y: 3057 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面8',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 0,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 6189   - 4000,
            y: 2291 - 2830
          },
          {
            x: 7219   - 4000,
            y: 2291 - 2830
          },
          {
            x: 7219   - 4000,
            y: 2346 - 2830
          },
          {
            x: 6189   - 4000,
            y: 2346 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面9',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 270,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 5061 - 4000,
            y: 2488 - 2830
          },
          {
            x: 5116 - 4000,
            y: 2488 - 2830
          },
          {
            x: 5116 - 4000,
            y: 3083 - 2830
          },
          {
            x: 5061 - 4000,
            y: 3083 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面10',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 0,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 3259  - 4000,
            y: 3097 - 2830
          },
          {
            x: 5006  - 4000,
            y: 3097 - 2830
          },
          {
            x: 5006  - 4000,
            y: 3210 - 2830
          },
          {
            x: 3259  - 4000,
            y: 3210 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面11',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 180,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 4354 - 4000,
            y: 4097 - 2830
          },
          {
            x: 5020 - 4000,
            y: 4097 - 2830
          },
          {
            x: 5020 - 4000,
            y: 4152 - 2830
          },
          {
            x: 4354 - 4000,
            y: 4152 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面12',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 90,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 4273 - 4000,
            y: 3936 - 2830
          },
          {
            x: 4323 - 4000,
            y: 3936 - 2830
          },
          {
            x: 4323 - 4000,
            y: 4296 - 2830
          },
          {
            x: 4273 - 4000,
            y: 4296 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面13',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 180,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 3193 - 4000,
            y: 4241 - 2830
          },
          {
            x: 4273 - 4000,
            y: 4241 - 2830
          },
          {
            x: 4273 - 4000,
            y: 4296 - 2830
          },
          {
            x: 3193 - 4000,
            y: 4296 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面14',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 0,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 3193 - 4000,
            y: 3936 - 2830
          },
          {
            x: 4273 - 4000,
            y: 3936 - 2830
          },
          {
            x: 4273 - 4000,
            y: 3991 - 2830
          },
          {
            x: 3193 - 4000,
            y: 3991 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面15',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 270,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 3109 - 4000,
            y: 3936 - 2830
          },
          {
            x: 3164 - 4000,
            y: 3936 - 2830
          },
          {
            x: 3164 - 4000,
            y: 4897 - 2830
          },
          {
            x: 3109 - 4000,
            y: 4897 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '作业面16',
        Angle: 0,
      Type: [1, 2, 3, 4],
      Direction: 0,
        Color: '#00ff0080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 5063 - 4000,
            y: 2364 - 2830
          },
          {
            x: 5611 - 4000,
            y: 2291 - 2830
          },
          {
            x: 6114 - 4000,
            y: 2291 - 2830
          },
          {
            x: 6114 - 4000,
            y: 2346 - 2830
          },
          {
            x: 5622 - 4000,
            y: 2346 - 2830
          },
          {
            x: 5137 - 4000,
            y: 2475 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      },
    ];
    const tempCarAreaData = [
      {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '禁停区17',
        Angle: 0,
        Type: [1, 2, 3, 4],
      Direction: 0,
        Color: '#ff000080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 6195 - 4000,
            y: 3196 - 2830
          },
          {
            x: 6910 - 4000,
            y: 3196 - 2830
          },
          {
            x: 6910 - 4000,
            y: 4047 - 2830
          },
          {
            x: 6195 - 4000,
            y: 4047 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      }, {
        Id: '5f6719a4e03abe3856d87d7b',
        Name: '禁停区18',
        Angle: 0,
        Type: [1],
        Direction: 0,
        Color: '#ff000080',
        Point: {
          x: 0,
          y: 0
        },
        Width: 32,
        Border: {
          w: 0,
          x: 0,
          y: 0,
          z: 0
        },
        Height: 32,
        Enabled: true,
        FloorId: '5f670bb79e91f4410005f564',
        DrawMode: 0,
        FillMode: 1,
        GameMode: 0,
        ImageUrl: '/api/Objects/PlanPlatform/5e7b384e8121c2286c0cb290.png',
        CompanyId: '5f67042b9e91f4410005f55c',
        FixedSize: false,
        CanConnection: true,
        ConnectionObjs: [],
        Thickness: 0,
        BuildingId: '5f6704859e91f4410005f55f',
        MultiPoint: [
          {
            x: 5149 - 4000,
            y: 4097 - 2830
          },
          {
            x: 6094 - 4000,
            y: 4097 - 2830
          },
          {
            x: 6094 - 4000,
            y: 4152 - 2830
          },
          {
            x: 5149 - 4000,
            y: 4152 - 2830
          },
        ],
        TemplateId: '5e7b38610137e12020bd7f7b',
        FireElementId: '5e7967f30814180f1c1ecaab',
        PropertyInfos: [],
        IsFromBuilding: true,
        InteractiveMode: 0
      }
    ];

    putCarAreaData.forEach(element => {
      const poly = new PutCarArea(element, this);
      poly.visible = false;
    });

    tempCarAreaData.forEach(element => {
      const poly = new PutCarArea(element, this);
      poly.visible = false;
      poly.polygonGraphics.interactive = false;
    });




    this.setNameVisible(false, GameMode.BasicInformation);
  }
  /**
   * 显示车辆图标
   * @param data
   */
  public ShowCarIcon(data: any) {

    this.carData.set(data.Id, data);

    const carButton = PIXI.Sprite.from(data.ImageUrl);
    carButton.name = data.Id;
    carButton.buttonMode = true;
    carButton.interactive = true;
    carButton
      .on('pointerdown', (event) => {
        if (this.paintMode !== PaintMode.Car) {
          this.setPaintMode(PaintMode.Car);
          this.selectCar = this.carData.get(event.currentTarget.name);
          event.currentTarget.name = '';
          event.currentTarget.visible = false;
          this.previewSinglePointIcon.texture = PIXI.Texture.from(data.ImageUrl);
          this.previewSinglePointIcon.visible = true;
          this.previewSinglePointIcon.scale.set(1 / this.backgroundImage.scale.x);
          this.backgroundImage.children.forEach(item => {
            if (item instanceof PutCarArea) {
              if (item.assetData.Type.indexOf(this.selectCar.Type) !== -1) {
                item.visible = true;
              } else {
                item.visible = false;
              }
            }
          });
        }
      })
      .on('pointerup', (event) => {

      })
      .on('pointerupoutside', (event) => {

      })
      .on('pointerover', (event) => {

      })
      .on('pointerout', (event) => {

      });


    carButton.setParent(this.app.stage);
    carButton.position.set(25, this.count * 65);
    this.count++;
  }
  /**
   *
   * @param id 图标ID
   * @param b 显示/隐藏
   */
  public setIconVisible(ids: string[], b: boolean) {
    ids.forEach(item => {
      this.backgroundImage.getChildByName(item).visible = b;
    });
  }
  /**
   * 版本检查
   */
  public versionChecking(): void {
    const floorData = this.canvasData.originaleveryStoreyData;
    const buildingData = this.canvasData.originalcompanyBuildingData;
    if (floorData.version && floorData.version === '1.0') {
      floorData.version = '2.0';
      Object.keys(floorData.data).forEach(item => {
        floorData.data[item].Point.y *= -1;
        floorData.data[item].MultiPoint?.forEach(element => {
          element.y *= -1;
        });
      });
    }
    if (buildingData.version && buildingData.version === '1.0') {
      buildingData.version = '2.0';
      Object.keys(buildingData.data).forEach(item => {
        buildingData.data[item].Point.y *= -1;
        buildingData.data[item].MultiPoint?.forEach(element => {
          element.y *= -1;
        });
      });
    }
  }
  /**
   * 创建确认绘制结束按钮
   */
  private createEnterPaintEndButton() {
    this.enterPaintEndButton.width = 60;
    this.enterPaintEndButton.height = 60;
    this.enterPaintEndButton.anchor.set(0.5);
    this.enterPaintEndButton.position =  new PIXI.Point(0, 0);
    this.enterPaintEndButton.interactive = true;
    this.enterPaintEndButton.buttonMode = true;
    this.enterPaintEndButton
      .on('mousedown', event => {
        event.stopPropagation();
        this.emit('createIcon');
        this.enterPaintEndButton.visible = false;
        this.enterPaint();
      });
    this.backgroundImage.addChild(this.enterPaintEndButton);
    this.enterPaintEndButton.zIndex = this.backgroundImage.children.length;
    this.enterPaintEndButton.visible = false;
  }
  // /**
  //  * 刷新确认按钮
  //  */
  // private refreshEnterPaintEndButton() {
  //   this.enterPaintEndButton.position = this.circleShadow.position;
  //   this.enterPaintEndButton.visible = true;
  // }
  /**
   * 创建背景图
   */
  private createBackgroundImage(): void {
    this.backgroundImage = PIXI.Sprite.from('assets/images/noImg.png');
    this.backgroundImage.anchor.set(0.5);
    this.backgroundImage.x = this.app.view.width / 2;
    this.backgroundImage.y = this.app.view.height / 2;
    this.backgroundImage.interactive = true;
    this.backgroundImage.name = 'background';
    this.backgroundImage.sortableChildren = true;
    this.backgroundImage
      .on('mousedown', event => {
        if (!event.currentTarget.dragging && this.selection.isMultiselection === false) {
          event.currentTarget.data = event.data;
          event.currentTarget.dragging = true;
          event.currentTarget.dragPoint = event.data.getLocalPosition(event.currentTarget.parent);
          event.currentTarget.dragPoint.x -= event.currentTarget.x;
          event.currentTarget.dragPoint.y -= event.currentTarget.y;
          switch (this.paintMode) {
            case PaintMode.endPaint:
              console.log(this.backgroundImage.toLocal(this.mousePosition));
              break;
            case PaintMode.singlePointIcon:
              const json = JSON.parse(JSON.stringify(this.canvasData.selectTemplateData.propertyInfos));
              const list = [];
              json.forEach(element => {
                const property = new PropertyInfo(element);
                list.push(property);
              });
              const assetData = {
                TemplateId: this.canvasData.selectTemplateData.id,
                FloorId: this.canvasData.selectStorey.id,
                Angle: this.canvasData.selectTemplateData.angle,
                Color: this.canvasData.selectTemplateData.color,
                Enabled: this.canvasData.selectTemplateData.enabled,
                FillMode: this.canvasData.selectTemplateData.fillMode,
                FireElementId: this.canvasData.selectTemplateData.fireElementId,
                FixedSize: this.canvasData.selectTemplateData.fixedSize,
                Height : 32,
                Width : 32,
                Id: ObjectID.default.generate(),
                ImageUrl: this.canvasData.selectTemplateData.imageUrl,
                InteractiveMode: this.canvasData.selectTemplateData.interactiveMode,
                MultiPoint : null,
                Point: new PIXI.Point(this.previewSinglePointIcon.x, this.previewSinglePointIcon.y),
                Name : this.canvasData.selectTemplateData.name,
                PropertyInfos: list,
                Border : this.canvasData.selectTemplateData.border,
                DrawMode : this.canvasData.selectTemplateData.drawMode,
                Thickness : this.canvasData.selectTemplateData.thickness,
                IsFromBuilding : this.canvasData.selectTemplateData.isFromBuilding,
                GameMode : this.canvasData.gameMode
              };
              const singleIcon = new SinglePointIcon(assetData, this);
              if (singleIcon.assetData.IsFromBuilding) {
                this.canvasData.originalcompanyBuildingData.data[singleIcon.assetData.Id] = singleIcon.assetData;
              } else {
                this.canvasData.originaleveryStoreyData.data[singleIcon.assetData.Id] = singleIcon.assetData;
              }
              this.emit('createIcon');
              this.emit('backgroundScale', this.backgroundImage.scale.x);
              this.canvasData.isChange = true;
              break;
            case PaintMode.lineIcon:
              this.previewLineSegment.visible = true;
              this.currentClickPoint.position = new PIXI.Point(this.circleShadow.x, this.circleShadow.y);
              this.paintPoints.push(new PIXI.Point(this.circleShadow.x, this.circleShadow.y));

              if (this.paintPoints.length >= 2) {
                this.enterPaintEndButton.position = this.circleShadow.position;
                this.enterPaintEndButton.visible = true;
              }

              if (this.paintingIcon !== null) {
                this.backgroundImage.removeChild(this.paintingIcon);
              }
              const jsonObject = JSON.parse(JSON.stringify(this.canvasData.selectTemplateData.propertyInfos));
              const propertyList = [];
              jsonObject.forEach(element => {
                const property = new PropertyInfo(element);
                propertyList.push(property);
              });
              const assetData1 = {
                TemplateId: this.canvasData.selectTemplateData.id,
                FloorId: this.canvasData.selectStorey.id,
                Angle: this.canvasData.selectTemplateData.angle,
                Color: this.canvasData.selectTemplateData.color,
                Enabled: this.canvasData.selectTemplateData.enabled,
                FillMode: this.canvasData.selectTemplateData.fillMode,
                FireElementId: this.canvasData.selectTemplateData.fireElementId,
                FixedSize: this.canvasData.selectTemplateData.fixedSize,
                Height: 32,
                Width: 32,
                Id: ObjectID.default.generate(),
                ImageUrl: this.canvasData.selectTemplateData.imageUrl,
                InteractiveMode: this.canvasData.selectTemplateData.interactiveMode,
                MultiPoint: JSON.parse(JSON.stringify(this.paintPoints)),
                Point: new PIXI.Point(0, 0),
                Name: this.canvasData.selectTemplateData.name,
                PropertyInfos: propertyList,
                Border: this.canvasData.selectTemplateData.border,
                DrawMode: this.canvasData.selectTemplateData.drawMode,
                Thickness: this.canvasData.selectTemplateData.thickness,
                IsFromBuilding: this.canvasData.selectTemplateData.isFromBuilding,
                GameMode: this.canvasData.gameMode
              };
              // const assetData1 = {
              //   ImageUrl: this.canvasData.selectTemplateData.imageUrl,
              //   Point: new PIXI.Point(0, 0),
              //   Width: 32,
              //   Height: 32,
              //   MultiPoint: this.paintPoints,
              //   Name: this.canvasData.selectTemplateData.name
              // };
              this.paintingIcon = new MultipointIcon(assetData1, this);
              // this.paintingIcon = new MultipointIcon(this.previewSinglePointIcon.texture, new PIXI.Point(0, 0), this.paintPoints, this,
              //   this.canvasData.selectTemplateData.name);
              this.emit('backgroundScale', this.backgroundImage.scale.x);
              break;
            case PaintMode.polygonIcon:
              this.previewLineSegment.visible = true;
              this.currentClickPoint.position = new PIXI.Point(this.circleShadow.x, this.circleShadow.y);
              this.paintPoints.push(new PIXI.Point(this.circleShadow.x, this.circleShadow.y));
              if (this.paintPoints.length === 1) {
                this.enterPaintEndButton.position = this.circleShadow.position;
              } else if (this.paintPoints.length >= 3) {
                this.enterPaintEndButton.visible = true;
              }
              this.paintPoints.forEach((value, index, array) => {
                if (index === 0) {
                  this.paintingLine.clear();
                  this.paintingLine.lineStyle(1, 0xffd900, 1);
                  this.paintingLine.moveTo(value.x, value.y);
                } else {
                  this.paintingLine.lineTo(value.x, value.y);
                }
              });

              // if (this.paintingIcon !== null) {
              //   this.backgroundImage.removeChild(this.paintingIcon);
              // }
              // this.paintingIcon = new PolygonIcon(this.paintPoints, this);
              break;
            case PaintMode.Pipeline:
              if (this.paintingPipeline !== null) {
                this.currentClickPoint.position = new PIXI.Point(this.circleShadow.x, this.circleShadow.y);
                this.paintPoints.push(new PIXI.Point(this.circleShadow.x, this.circleShadow.y));
                this.paintingPipeline.assetData.MultiPoint = JSON.parse(JSON.stringify(this.paintPoints));
                this.paintingPipeline.refresh();
              }
              // this.emit('backgroundScale', this.backgroundImage.scale.x);
              break;
            case PaintMode.Arrows:
              if (this.paintingArrows === null) {
                const data = {
                   Id: ObjectID.default.generate(),
                   name: 'string',
                   point: new PIXI.Point(this.circleShadow.x, this.circleShadow.y),
                   pointA: new PIXI.Point(this.circleShadow.x, this.circleShadow.y),
                   pointB: new PIXI.Point(this.circleShadow.x, this.circleShadow.y),
                   source: 'assets/images/进攻方向.png',
                };
                this.paintingArrows = new Arrows(data, this);
              } else {
                this.paintingArrows.ready = true;
                this.paintingArrows = null;
                this.paintMode = PaintMode.endPaint;
              }
              break;
            case PaintMode.Car:
              // console.log('创建车辆');
              break;
          }
        } else if (!event.currentTarget.dragging && this.selection.isMultiselection === true) {
          this.rectToolGraphics.visible = true;
          event.currentTarget.dragging = true;
          this.initialScreenMousePos = this.backgroundImage.toLocal(this.mousePosition);
          this.finalScreenMousePos = this.backgroundImage.toLocal(this.mousePosition);
        }
      })
      .on('mouseup', event => {
        if (event.currentTarget.dragging) {
          event.currentTarget.dragging = false;
          event.currentTarget.data = null;
        }
        if (this.rectToolGraphics.visible === true) {
          this.backgroundImage.children.forEach(item => {
            if (item instanceof SinglePointIcon
              || item instanceof MultipointIcon
              || item instanceof PolygonIcon) {
              if (this.rectToolGraphics.getLocalBounds().contains(item.x, item.y)) {
                this.selection.select(item);
              }
            }
          });
          this.rectToolGraphics.clear();
          this.rectToolGraphics.visible = false;
        }
      })
      .on('mouseupoutside', event => {
        if (event.currentTarget.dragging) {
          event.currentTarget.dragging = false;
          event.currentTarget.data = null;
        }
      })
      .on('mousemove', event => {
        if (event.currentTarget.dragging && this.selection.isMultiselection === false) {
          const newPosition = event.currentTarget.data.getLocalPosition(event.currentTarget.parent);
          event.currentTarget.x = newPosition.x - event.currentTarget.dragPoint.x;
          event.currentTarget.y = newPosition.y - event.currentTarget.dragPoint.y;
        } else if (event.currentTarget.dragging && this.selection.isMultiselection === true) {
          if (this.rectToolGraphics.visible === true) {
            this.finalScreenMousePos = this.backgroundImage.toLocal(this.mousePosition);
          }
        }
      }).on('rightclick', event => {
        event.stopPropagation();
        this.cancelPaint();
        this.selection.deselectAll();
      })
      .on('pointerover', (event) => {
        this.previewSinglePointIcon.filters = [new OldFilmFilter()];
      })
      .on('pointerout', (event) => {
        this.previewSinglePointIcon.filters = null;
      });
    this.app.stage.addChild(this.backgroundImage);

    this.createPreviewSinglePointIcon();
    this.createPreviewLineSegment();
    this.createCircleShadow();
    this.createEnterPaintEndButton();
    this.backgroundImage.addChild(this.paintingLine);
  }

  /**
   * 刷新背景图
   */
  public refreshBackgroundImage(): void {
    if (!this.canvasData.selectStorey.imageUrl) {
      this.backgroundImage.visible = false;
    } else {
      this.backgroundImage.texture = PIXI.Texture.from(this.canvasData.selectStorey.imageUrl);
      this.backgroundImage.angle = this.canvasData.selectStorey.imageAngle;
      this.backgroundImage.visible = true;
    }
  }
  /**
   * 清空画布
   */
  public destroyBackgroundImage(): void {
    this.app.stage.removeChild(this.backgroundImage);
    // this.removeAllListeners('backgroundScale');
  }
  /**
   * 设置背景图缩放
   * @param scale 缩放系数
   */
  public setBackgroundScale(scale: number): void {
    this.backgroundImage.scale.set(scale);
    this.emit('backgroundScale', this.backgroundImage.scale.x);
  }
  /**
   * 设置背景图角度
   * @param imageAngle 角度值
   */
  public setBackgroundAngle(imageAngle: number) {
    this.backgroundImage.angle = imageAngle;
  }
  /**
   * 创建预览单点图标
   */
  private createPreviewSinglePointIcon(): void {
    this.previewSinglePointIcon = PIXI.Sprite.from('assets/images/noImg.png');
    this.previewSinglePointIcon.width = 32;
    this.previewSinglePointIcon.height = 32;
    this.previewSinglePointIcon.alpha = 1;
    this.previewSinglePointIcon.anchor.set(0.5);
    this.previewSinglePointIcon.visible = false;
    this.backgroundImage.addChild(this.previewSinglePointIcon);
  }
  /**
   * 改变预览单点图标
   * @param uri 图片地址
   */
  private changePreviewSinglePointIcon(uri: string): void {
    this.previewSinglePointIcon.texture = PIXI.Texture.from(uri);
    this.previewSinglePointIcon.visible = true;
  }
  /**
   * 创建预览线段
   */
  private createPreviewLineSegment() {
    this.previewLineSegment.visible = false;
    this.backgroundImage.addChild(this.currentClickPoint);
    this.backgroundImage.addChild(this.previewLineSegment);

    this.backgroundImage.addChild(this.rectToolGraphics);
    this.rectToolGraphics.visible = false;
  }
  /**
   * 刷新预览线段
   * @param pointA 点A
   * @param pointB 点B
   */
  private refreshPreviewLineSegment(pointA: PIXI.Point, pointB: PIXI.Point) {
    this.previewLineSegment.clear();
    this.previewLineSegment.lineStyle(1, 0xffd900, 1);
    this.previewLineSegment.moveTo(pointA.x, pointA.y);
    this.previewLineSegment.lineTo(pointB.x, pointB.y );
  }
  /**
   * 创建半径图标影子
   * @param x 半径
   */
  private createCircleShadow(): void {
    this.circleShadow.beginFill(0xFFCC5A);
    this.circleShadow.drawCircle(0, 0, 10);
    this.circleShadow.endFill();
    this.circleShadow.visible = false;
    this.backgroundImage.addChild(this.circleShadow);
  }
  /**
   * 开始绘制
   */
  public beginPaint() {
    switch (this.canvasData.selectTemplateData.interactiveMode) {
      case 0:
        this.beginPaintSinglePointIcon();
        break;
      case 1:
        this.beginPaintLineIcon();
        break;
      case 2:
        this.beginPaintPolygonIcon();
        break;
    }
  }
  /**
   * 开始绘画单点图标
   */
  private beginPaintSinglePointIcon(): void {
    this.cancelPaint();
    this.paintMode = PaintMode.singlePointIcon;
    this.changePreviewSinglePointIcon(this.canvasData.selectTemplateData.imageUrl);
  }
  /**
   * 开始绘画多边形
   */
  private beginPaintPolygonIcon(): void {
    this.cancelPaint();
    this.paintMode = PaintMode.polygonIcon;
    this.circleShadow.visible = true;
  }
  /**
   * 开始绘制多点图标
   */
  private beginPaintLineIcon(): void {
    this.cancelPaint();
    this.paintMode = PaintMode.lineIcon;
    this.previewSinglePointIcon.texture = PIXI.Texture.from(this.canvasData.selectTemplateData.imageUrl);
    this.circleShadow.visible = true;
  }
  /**
   * 开始绘制管线
   */
  public beginPaintPipeline(): void {
    this.paintMode = PaintMode.Pipeline;
  }
  public paintingPipelineFinish(): void {
    if (this.paintMode === PaintMode.Pipeline) {
      this.paintMode = PaintMode.endPaint;
      this.paintPoints.splice(0, this.paintPoints.length);
      this.paintingPipeline = null;
    }
  }
  public beginPaintingArrows(): void {
    this.paintMode = PaintMode.Arrows;
  }


  /**
   * 取消绘画
   */
  public cancelPaint(): void {
    switch (this.paintMode) {
      case PaintMode.singlePointIcon:
        this.previewSinglePointIcon.visible = false;
        this.paintMode = PaintMode.endPaint;
        break;
      case PaintMode.lineIcon:
        this.circleShadow.visible = false;
        this.previewLineSegment.visible = false;
        this.paintPoints.splice(0, this.paintPoints.length);
        if (this.paintingIcon !== null) {
          this.backgroundImage.removeChild(this.paintingIcon);
        }
        this.paintMode = PaintMode.endPaint;
        break;
      case PaintMode.polygonIcon:
        this.circleShadow.visible = false;
        this.previewLineSegment.visible = false;
        this.paintingIcon = null;
        this.paintPoints.splice(0, this.paintPoints.length);
        this.paintingLine.clear();
        this.paintMode = PaintMode.endPaint;
        break;
    }
  }
  /**
   * 设置绘制状态
   * @param mode 状态
   */
  public setPaintMode(mode: PaintMode) {
    this.reset();
    this.paintMode = mode;
    switch (mode) {
      case PaintMode.endPaint:
        this.selectCar = null;
        this.backgroundImage.children.forEach(item => {
          if (item instanceof PutCarArea) {
            if (item.assetData.Type.indexOf(this.selectCar?.Type) !== -1) {
              item.visible = true;
            } else {
              item.visible = false;
            }
          }
        });
        break;
    }
  }
  /**
   * 获取绘制状态
   */
  public getPaintMode(): PaintMode {
    return this.paintMode;
  }
  /**
   * 重置
   */
  public reset() {
    this.previewSinglePointIcon.filters = null;
    this.previewSinglePointIcon.visible = false;
    this.previewSinglePointIcon.angle = 0;
  }
  /**
   * 确认绘制
   */
  private enterPaint(): void {
    this.previewLineSegment.visible = false;
    switch (this.paintMode) {
      case PaintMode.lineIcon:
        if (this.paintPoints.length >= 2) {

          if (this.paintingIcon.assetData.IsFromBuilding) {
            this.canvasData.originalcompanyBuildingData.data[this.paintingIcon.assetData.Id] = this.paintingIcon.assetData;
          } else {
            this.canvasData.originaleveryStoreyData.data[this.paintingIcon.assetData.Id] = this.paintingIcon.assetData;
          }
          this.canvasData.isChange = true;
          this.paintingIcon = null;
        }
        break;
      case PaintMode.polygonIcon:
        this.paintingLine.clear();
        if (this.paintPoints.length >= 3) {
          const jsonList = JSON.parse(JSON.stringify(this.canvasData.selectTemplateData.propertyInfos));
          const propertyList = [];
          jsonList.forEach(element => {
            const property = new PropertyInfo(element);
            propertyList.push(property);
          });
          const assetData = {
            TemplateId: this.canvasData.selectTemplateData.id,
            FloorId: this.canvasData.selectStorey.id,
            Angle: this.canvasData.selectTemplateData.angle,
            Color: this.canvasData.selectTemplateData.color,
            Enabled: this.canvasData.selectTemplateData.enabled,
            FillMode: this.canvasData.selectTemplateData.fillMode,
            FireElementId: this.canvasData.selectTemplateData.fireElementId,
            FixedSize: this.canvasData.selectTemplateData.fixedSize,
            Height: 32,
            Width: 32,
            Id: ObjectID.default.generate(),
            ImageUrl: this.canvasData.selectTemplateData.imageUrl,
            InteractiveMode: this.canvasData.selectTemplateData.interactiveMode,
            MultiPoint: JSON.parse(JSON.stringify(this.paintPoints)),
            Point: new PIXI.Point(0, 0),
            Name: this.canvasData.selectTemplateData.name,
            PropertyInfos: propertyList,
            Border: this.canvasData.selectTemplateData.border,
            DrawMode: this.canvasData.selectTemplateData.drawMode,
            Thickness: this.canvasData.selectTemplateData.thickness,
            IsFromBuilding: this.canvasData.selectTemplateData.isFromBuilding,
            GameMode: this.canvasData.gameMode
          };
          const polygonIcon = new PolygonIcon(assetData, this);
          if (polygonIcon.assetData.IsFromBuilding) {
            this.canvasData.originalcompanyBuildingData.data[polygonIcon.assetData.Id] = polygonIcon.assetData;
          } else {
            this.canvasData.originaleveryStoreyData.data[polygonIcon.assetData.Id] = polygonIcon.assetData;
          }
          this.canvasData.isChange = true;
        }
        break;
    }
    this.paintPoints.splice(0, this.paintPoints.length);
    this.emit('backgroundScale', this.backgroundImage.scale.x);
  }
  /**
   * 复制
   */
  public copy(): void {
    this.copyData = [];
    this.selection.objects.forEach(item => {
      const newData = JSON.parse(JSON.stringify(item.assetData));
      this.copyData.push(newData);
    });
  }
  /**
   * 粘贴
   */
  public paste(companyId: string, buildingId: string, floorId: string): void {
    this.copyData.forEach(item => {
      item.Point = new PIXI.Point(item.Point.x + 5, item.Point.y + 5);
      const newData = JSON.parse(JSON.stringify(item));
      newData.Id = ObjectID.default.generate(),
      newData.CompanyId = companyId;
      newData.BuildingId = buildingId;
      newData.FloorId = floorId;
      newData.Point = new PIXI.Point(item.Point.x + 5, item.Point.y + 5);
      if (newData.IsFromBuilding) {
        this.canvasData.originalcompanyBuildingData.data[newData.Id] = newData;
      } else {
        this.canvasData.originaleveryStoreyData.data[newData.Id] = newData;
      }
      switch (item.InteractiveMode) {
        case PaintMode.singlePointIcon:
          const singleIcon = new SinglePointIcon(newData, this);
          break;
        case PaintMode.lineIcon:
          const lineIcon = new MultipointIcon(newData, this);
          break;
        case PaintMode.polygonIcon:
          const polygonIcon = new PolygonIcon(newData, this);
          break;
      }
      this.selection.select(this.backgroundImage.getChildByName(newData.Id));
    });
  }

}
/**
 * 绘制模式
 */
enum PaintMode {
  singlePointIcon,
  lineIcon,
  polygonIcon,
  endPaint,
  Pipeline,
  Arrows,
  Car
}
/**
 * 单点图标
 */
export class SinglePointIcon extends PIXI.Container {
  style = new PIXI.TextStyle({
    fontFamily: 'Arial',
    fontSize: 18,
    fontStyle: 'normal',
    fontWeight: 'bold',
    fill: ['#000000'],
    stroke: '#ffffff',
    strokeThickness: 3,
    dropShadow: true,
    dropShadowColor: '#000000',
    dropShadowBlur: 3,
    dropShadowAngle: Math.PI / 6,
    dropShadowDistance: 1,
    wordWrap: false,
    wordWrapWidth: 100,
  });

  public text = new PIXI.Text(this.assetData.Name
    + '\r\n'
    + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue, this.style);
  private image = PIXI.Sprite.from(this.assetData.ImageUrl);

  constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
    super();
    this.workingArea.backgroundImage.addChild(this);
    this.x = this.assetData.Point.x;
    this.y = this.assetData.Point.y;
    this.name = this.assetData.Id;

    this.image.angle = this.assetData.Angle;

    this.image.x = 0;
    this.image.y = 0;
    this.image.width = this.assetData.Width;
    this.image.height = this.assetData.Height;

    this.image.alpha = 1;
    this.image.anchor.set(0.5);
    this.image.interactive = true;
    this.image
      .on('mousedown', event => {
        console.log(this.assetData.Pipelines);
        event.stopPropagation();
        this.workingArea.selection.selectOne(this);
        if (this.assetData.CanConnection && this.parent === this.workingArea.backgroundImage) {
          if (this.workingArea.getPaintMode() === PaintMode.Pipeline) {
            if (this.workingArea.paintingPipeline === null) {
              this.workingArea.previewLineSegment.visible = true;
              this.workingArea.currentClickPoint.position =
                new PIXI.Point(this.workingArea.circleShadow.x, this.workingArea.circleShadow.y);
              this.workingArea.paintPoints.push(new PIXI.Point(this.position.x, this.position.y));
              const tempData = {
                Id: ObjectID.default.generate(),
                MultiPoint: JSON.parse(JSON.stringify(this.workingArea.paintPoints)),
                Point: new PIXI.Point(0, 0),
                Name: '管线',
                LinkedObjects: [],
              };
              this.workingArea.paintingPipeline = new Pipeline(tempData, this.workingArea);
              this.workingArea.paintingPipeline.assetData.LinkedObjects.push(this);
              this.assetData.Pipelines.push(this.workingArea.paintingPipeline);
            } else {
              this.workingArea.previewLineSegment.visible = false;
              this.workingArea.currentClickPoint.position =
                new PIXI.Point(this.workingArea.circleShadow.x, this.workingArea.circleShadow.y);
              this.workingArea.paintPoints.push(new PIXI.Point(this.position.x, this.position.y));
              this.workingArea.paintingPipeline.assetData.MultiPoint = JSON.parse(JSON.stringify(this.workingArea.paintPoints));
              this.workingArea.paintingPipeline.assetData.LinkedObjects.push(this);
              this.assetData.Pipelines.push(this.workingArea.paintingPipeline);
              this.workingArea.paintingPipeline.refresh();
              this.workingArea.paintingPipelineFinish();
            }
          }
        }
        // 如果链接对象不为空,禁止移动
        if ((this.workingArea.allowEdit || this.parent !== this.workingArea.backgroundImage) ) {
          event.currentTarget.parent.data = event.data;
          event.currentTarget.parent.alpha = 0.5;
          event.currentTarget.parent.dragging = true;
        }
     })
     .on('mouseup', event => {
       if (event.currentTarget.parent.dragging) {
         event.currentTarget.parent.alpha = 1;
         event.currentTarget.parent.dragging = false;
         event.currentTarget.parent.data = null;
       }
     })
      .on('mouseupoutside', event => {
        if (event.currentTarget.parent.dragging) {
          event.currentTarget.parent.alpha = 1;
          event.currentTarget.parent.dragging = false;
          event.currentTarget.parent.data = null;
        }
     })
     .on('mousemove', event => {
       if (event.currentTarget.parent.dragging) {
         // 如果拖动过程中发现父对象不是背景图
         if (this.parent !== this.workingArea.backgroundImage) {
           this.setParent(this.workingArea.backgroundImage);
           if (this.assetData.FixedSize) {
                const scale = 1 / this.workingArea.backgroundImage.scale.x;
                this.scale.set(scale);
            }
         }
         const newPosition = event.currentTarget.parent.data.getLocalPosition(event.currentTarget.parent.parent);
         event.currentTarget.parent.x = newPosition.x;
         event.currentTarget.parent.y = newPosition.y;
         this.assetData.Point = new PIXI.Point(this.x, this.y);
         this.workingArea.canvasData.isChange = true;
       }
     })
     .on('rightclick', event => {

     })
     .on('mouseover', event => {

     });

    // this.workingArea.on('backgroundScale', data => {
    //   if (this.assetData.FixedSize) {
    //     const scale = 1 / data;
    //     this.scale.set(scale);
    //   }
    // });

    this.text.x = this.image.x;
    this.text.y = this.image.y - this.image.height / 2;
    this.text.anchor.set(0.5, 1);

    if (this.assetData.GameMode === 2) {
      this.text.visible = false;
    }

    this.addChild(this.text);
    this.addChild(this.image);
  }
  // 设置名称
  public setNameVisible(value: boolean, mode: GameMode) {
    if (this.assetData.GameMode === mode) {
      this.text.visible = value;
    }
  }
  // 刷新
  public refresh() {
    this.image.width = this.assetData.Width;
    this.image.height = this.assetData.Height;
    this.image.angle = this.assetData.Angle;
    this.text.text = this.assetData.Name
      + '\r\n'
      + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue;
    this.text.x = this.image.x;
    this.text.y = this.image.y - this.image.height / 2;
  }
}
/**
 * 多点连线
 */
export class MultipointIcon extends PIXI.Container {
  public pointsData: PIXI.Point[];
  public pointsGraphics: PIXI.Graphics[] = [];
  public iconsTilingSprite: PIXI.TilingSprite[] = [];
  style = new PIXI.TextStyle({
    fontFamily: 'Arial',
    fontSize: 18,
    fontStyle: 'normal',
    fontWeight: 'bold',
    fill: ['#000000'],
    stroke: '#ffffff',
    strokeThickness: 3,
    dropShadow: true,
    dropShadowColor: '#000000',
    dropShadowBlur: 3,
    dropShadowAngle: Math.PI / 6,
    dropShadowDistance: 1,
    wordWrap: false,
    wordWrapWidth: 100,
  });

  public text = new PIXI.Text(this.assetData.Name
    + '\r\n'
    + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue, this.style);
  /**
   *
   * @param texture 图片素材
   * @param points  点集合
   */
  constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
    super();
    this.name = this.assetData.Id;
    this.pointsData = this.assetData.MultiPoint;
    this.x = this.assetData.Point.x;
    this.y = this.assetData.Point.y;
    this.workingArea.backgroundImage.addChild(this);
    // 画线图标
    for (let i = 0, count = this.pointsData.length - 1; i < count; i++) {
      const pointA = this.pointsData[i];
      const pointB = this.pointsData[i + 1];

      const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
      const a = pointB.x - pointA.x;
      const b = pointB.y - pointA.y;
      const distance = Math.sqrt(a * a + b * b);

      const icon = new PIXI.TilingSprite(PIXI.Texture.from(this.assetData.ImageUrl), distance, 64);
      icon.anchor.set(0, 0.5);
      icon.x = pointA.x;
      icon.y = pointA.y;
      icon.angle = angle;
      this.iconsTilingSprite.push(icon);
      this.addChild(icon);
      if (i === 0) {
        this.text.anchor.set(0.5);
        this.text.position = icon.position;
        this.text.y -= this.assetData.Height;
        this.addChild(this.text);
      }
    }
    // 画点
    this.pointsData.forEach((item, index, array) => {
      const iconPoint = new PIXI.Graphics();
      iconPoint.lineStyle(1, 0xFFBD01, 1);
      iconPoint.beginFill(0xFFFFFF, 1);
      iconPoint.drawCircle(0, 0, 15);
      iconPoint.x = item.x;
      iconPoint.y = item.y;
      iconPoint.endFill();
      iconPoint.visible = false;
      this.pointsGraphics.push(iconPoint);
      this.addChild(iconPoint);
    });
    // 添加圆点事件
    this.pointsGraphics.forEach((item, index, array) => {
      item.interactive = true;
      item.on('mousedown', event => {
        event.stopPropagation();
        if (this.workingArea.allowEdit) {
          event.currentTarget.data = event.data;
          event.currentTarget.alpha = 0.5;
          event.currentTarget.dragging = true;
        }
      })
        .on('mouseup', event => {
          if (event.currentTarget.dragging) {
            event.currentTarget.alpha = 1;
            event.currentTarget.dragging = false;
            event.currentTarget.data = null;
          }
        })
        .on('mouseupoutside', event => {
          if (event.currentTarget.dragging) {
            event.currentTarget.alpha = 1;
            event.currentTarget.dragging = false;
            event.currentTarget.data = null;
          }
        })
        .on('mousemove', event => {
          if (event.currentTarget.dragging) {
            const newPosition = event.currentTarget.data.getLocalPosition(event.currentTarget.parent);
            event.currentTarget.x = newPosition.x;
            event.currentTarget.y = newPosition.y;

            this.assetData.MultiPoint[index].x = newPosition.x;
            this.assetData.MultiPoint[index].y = newPosition.y;
            this.workingArea.canvasData.isChange = true;

            if (index === 0) {// 第一个点
              this.iconsTilingSprite[index].x = newPosition.x;
              this.iconsTilingSprite[index].y = newPosition.y;

              const pointA = array[index];
              const pointB = array[index + 1];

              const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
              const a = pointB.x - pointA.x;
              const b = pointB.y - pointA.y;
              const distance = Math.sqrt(a * a + b * b);
              this.iconsTilingSprite[index].angle = angle;
              this.iconsTilingSprite[index].width = distance;

              this.text.position = this.iconsTilingSprite[index].position;
              this.text.y -= this.assetData.Height;
            } else if (index < array.length - 1) {// 不是第一个点，也不是最后一个点
              this.iconsTilingSprite[index].x = newPosition.x;
              this.iconsTilingSprite[index].y = newPosition.y;

              const pointA = array[index]; // 当前点
              const pointB = array[index + 1]; // 后一个点
              const pointC = array[index - 1]; // 前一个点

              const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
              const a = pointB.x - pointA.x;
              const b = pointB.y - pointA.y;
              const distance = Math.sqrt(a * a + b * b);
              this.iconsTilingSprite[index].angle = angle;
              this.iconsTilingSprite[index].width = distance;

              const angleC = Math.atan2((pointA.y - pointC.y), (pointA.x - pointC.x)) * (180 / Math.PI);
              const aC = pointA.x - pointC.x;
              const bC = pointA.y - pointC.y;
              const distanceC = Math.sqrt(aC * aC + bC * bC);
              this.iconsTilingSprite[index - 1].angle = angleC;
              this.iconsTilingSprite[index - 1].width = distanceC;
            } else if (index === array.length - 1) { // 最后一个点
              const pointA = array[index]; // 当前点
              const pointC = array[index - 1]; // 前一个点

              const angleC = Math.atan2((pointA.y - pointC.y), (pointA.x - pointC.x)) * (180 / Math.PI);
              const aC = pointA.x - pointC.x;
              const bC = pointA.y - pointC.y;
              const distanceC = Math.sqrt(aC * aC + bC * bC);
              this.iconsTilingSprite[index - 1].angle = angleC;
              this.iconsTilingSprite[index - 1].width = distanceC;
            }
          }
        })
        .on('rightclick', event => {
        })
        .on('mouseover', event => {

        });
    });
    // // 缩放
    // this.workingArea.on('backgroundScale', data => {
    //   const scale = 1 / data;
    //   this.text.scale.set(scale);
    // });
    // 添加选中事件
    this.iconsTilingSprite.forEach((item, index, array) => {
      item.interactive = true;
      item.on('mousedown', event => {
        event.stopPropagation();
        this.workingArea.selection.selectOne(this);
        if (this.workingArea.allowEdit) {
          event.currentTarget.parent.data = event.data;
          event.currentTarget.parent.alpha = 0.5;
          event.currentTarget.parent.dragging = true;

          event.currentTarget.parent.dragPoint = event.data.getLocalPosition(event.currentTarget.parent.parent);
          event.currentTarget.parent.dragPoint.x -= event.currentTarget.parent.x;
          event.currentTarget.parent.dragPoint.y -= event.currentTarget.parent.y;
        }
      })
        .on('mouseup', event => {
          if (event.currentTarget.parent.dragging) {
            event.currentTarget.parent.alpha = 1;
            event.currentTarget.parent.dragging = false;
            event.currentTarget.parent.data = null;
          }
        })
        .on('mouseupoutside', event => {
          if (event.currentTarget.parent.dragging) {
            event.currentTarget.parent.alpha = 1;
            event.currentTarget.parent.dragging = false;
            event.currentTarget.parent.data = null;
          }
        })
        .on('mousemove', event => {
          if (event.currentTarget.parent.dragging) {
            const newPosition = event.currentTarget.parent.data.getLocalPosition(event.currentTarget.parent.parent);
            event.currentTarget.parent.x = newPosition.x - event.currentTarget.parent.dragPoint.x;
            event.currentTarget.parent.y = newPosition.y - event.currentTarget.parent.dragPoint.y;

            this.assetData.Point = new PIXI.Point(this.x, this.y);
            this.workingArea.canvasData.isChange = true;
          }
        })
        .on('rightclick', event => {

        });
    });
  }
  /**
   * 设置点显示状态
   * @param value 显示状态
   */
  public setPointVisiable(value: boolean) {
    this.pointsGraphics.forEach((item) => {
      item.visible = value;
    });
  }
  // 设置名称
  public setNameVisible(value: boolean, mode: GameMode) {
    if (this.assetData.GameMode === mode) {
      this.text.visible = value;
    }
  }
  //
  public refresh() {
    this.text.text = this.assetData.Name
      + '\r\n'
      + this.assetData.PropertyInfos.find(item => item.PropertyName === '名称/编号')?.PropertyValue;
  }
}
/**
 * 多边形
 */
export class PolygonIcon extends PIXI.Container {
  public pointsData: PIXI.Point[];
  public pointsGraphics: PIXI.Graphics[] = [];
  public polygonGraphics: PIXI.Graphics = new PIXI.Graphics();
  public polygonLineGraphics: PIXI.Graphics = new PIXI.Graphics();
  style = new PIXI.TextStyle({
    fontFamily: 'Arial',
    fontSize: 18,
    fontStyle: 'normal',
    fontWeight: 'bold',
    fill: ['#000000'],
    stroke: '#ffffff',
    strokeThickness: 3,
    dropShadow: true,
    dropShadowColor: '#000000',
    dropShadowBlur: 3,
    dropShadowAngle: Math.PI / 6,
    dropShadowDistance: 1,
    wordWrap: false,
    wordWrapWidth: 100,
  });

  public text = new PIXI.Text(this.assetData.Name
    + '\r\n'
    + this.assetData.PropertyInfos.find(item => item.PropertyName === '名称/编号')?.PropertyValue, this.style);
  /**
   *
   * @param points  点集合
   */
  constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
    super();
    this.name = this.assetData.Id;
    this.x = this.assetData.Point.x;
    this.y = this.assetData.Point.y;
    this.pointsData = this.assetData.MultiPoint;
    this.workingArea.backgroundImage.addChild(this);
    this.sortableChildren = true;
    // 画点
    this.pointsData.forEach((item, index, array) => {
      const iconPoint = new PIXI.Graphics();
      iconPoint.lineStyle(1, 0xFFBD01, 1);
      iconPoint.beginFill(0xFFFFFF, 1);
      iconPoint.drawCircle(0, 0, 15);
      iconPoint.x = item.x;
      iconPoint.y = item.y;
      iconPoint.endFill();
      iconPoint.visible = false;
      this.pointsGraphics.push(iconPoint);
      this.addChild(iconPoint);
    });
    // 填充多边形

    const color: number = this.assetData.Color.substring(0, 7).replace('#', '0x');
    const angle: number = parseInt(this.assetData.Color.substring(7), 16) / 255;
    this.polygonGraphics.beginFill(color, angle);
    this.polygonGraphics.drawPolygon(this.getPoints());
    this.polygonGraphics.endFill();
    this.addChild(this.polygonGraphics);
    // 画多边形
    this.polygonLineGraphics.lineStyle(5, 0xFFBD01, 1);
    this.polygonLineGraphics.drawPolygon(this.getPoints());
    this.polygonLineGraphics.closePath();
    this.addChild(this.polygonLineGraphics);

    this.text.anchor.set(0.5);
    this.text.position = this.calculatePolygonGravityCenter(this.pointsData);
    // console.log(this.calculatePolygonGravityCenter(this.pointsData));
    this.polygonGraphics.addChild(this.text);
    // 添加圆点事件
    this.pointsGraphics.forEach((item, index, array) => {
      item.interactive = true;
      item.zIndex = 1;
      item.on('mousedown', event => {
        event.stopPropagation();
        if (this.workingArea.allowEdit) {
          event.currentTarget.data = event.data;
          event.currentTarget.alpha = 0.5;
          event.currentTarget.dragging = true;
        }
      })
        .on('mouseup', event => {
          if (event.currentTarget.dragging) {
            event.currentTarget.alpha = 1;
            event.currentTarget.dragging = false;
            event.currentTarget.data = null;
          }
        })
        .on('mouseupoutside', event => {
          if (event.currentTarget.dragging) {
            event.currentTarget.alpha = 1;
            event.currentTarget.dragging = false;
            event.currentTarget.data = null;
          }
        })
        .on('mousemove', event => {
          if (event.currentTarget.dragging) {
            const newPosition = event.currentTarget.data.getLocalPosition(event.currentTarget.parent);
            event.currentTarget.x = newPosition.x;
            event.currentTarget.y = newPosition.y;

            this.assetData.MultiPoint[index].x = newPosition.x;
            this.assetData.MultiPoint[index].y = newPosition.y;
            this.workingArea.canvasData.isChange = true;
            // 填充多边形
            this.polygonGraphics.clear();
            this.polygonGraphics.beginFill(color, angle);
            this.polygonGraphics.drawPolygon(this.getPoints());
            this.polygonGraphics.endFill();
            // 画多边形
            this.polygonLineGraphics.clear();
            this.polygonLineGraphics.lineStyle(5, 0xFFBD01, 1);
            this.polygonLineGraphics.drawPolygon(this.getPoints());
            this.polygonLineGraphics.closePath();

            this.text.position = this.calculatePolygonGravityCenter(this.pointsData);
          }
        })
        .on('rightclick', event => {
        });
    });
    // 添加选中事件
    this.polygonGraphics.interactive = true;
    this.polygonGraphics
      .on('mousedown', event => {
        event.stopPropagation();
        this.workingArea.selection.selectOne(this);
        if (this.workingArea.allowEdit) {
          event.currentTarget.parent.data = event.data;
          event.currentTarget.parent.alpha = 0.5;
          event.currentTarget.parent.dragging = true;

          event.currentTarget.parent.dragPoint = event.data.getLocalPosition(event.currentTarget.parent.parent);
          event.currentTarget.parent.dragPoint.x -= event.currentTarget.parent.x;
          event.currentTarget.parent.dragPoint.y -= event.currentTarget.parent.y;
        }
      })
      .on('mouseup', event => {
        if (event.currentTarget.parent.dragging) {
          event.currentTarget.parent.alpha = 1;
          event.currentTarget.parent.dragging = false;
          event.currentTarget.parent.data = null;
        }
      })
      .on('mouseupoutside', event => {
        if (event.currentTarget.parent.dragging) {
          event.currentTarget.parent.alpha = 1;
          event.currentTarget.parent.dragging = false;
          event.currentTarget.parent.data = null;
        }
      })
      .on('mousemove', event => {
        if (event.currentTarget.parent.dragging) {
          const newPosition = event.currentTarget.parent.data.getLocalPosition(event.currentTarget.parent.parent);
          event.currentTarget.parent.x = newPosition.x - event.currentTarget.parent.dragPoint.x;
          event.currentTarget.parent.y = newPosition.y - event.currentTarget.parent.dragPoint.y;

          this.assetData.Point = new PIXI.Point(this.x, this.y);
          this.workingArea.canvasData.isChange = true;
        }
      })
      .on('rightclick', event => {
        // this.workingArea.selection.deselectAll();
      });
    // // 缩放
    // this.workingArea.on('backgroundScale', data => {
    //   const scale = 1 / data;
    //   this.text.scale.set(scale);
    // });
  }
  /**
   * 设置点显示状态
   * @param value 显示状态
   */
  public setPointVisiable(value: boolean) {
    this.pointsGraphics.forEach((item) => {
      item.visible = value;
    });
  }

  public calculatePolygonGravityCenter(points: PIXI.Point[]) {
  let area = 0.0; // 多边形面积
  let gravityLat = 0.0; // 重心点 latitude
  let gravityLng = 0.0; // 重心点 longitude
  points.forEach((item, index) => {
    // 1
    const lat = item.x;
    const lng = item.y;
    const nextLat = points[(index + 1) % points.length].x;
    const nextLng = points[(index + 1) % points.length].y;
    // 2
    const tempArea = (nextLat * lng - nextLng * lat) / 2.0;
    // 3
    area += tempArea;
    // 4
    gravityLat += tempArea * (lat + nextLat) / 3;
    gravityLng += tempArea * (lng + nextLng) / 3;
    });
  // 5
  gravityLat = gravityLat / area;
  gravityLng = gravityLng / area;

  return new PIXI.Point(gravityLat, gravityLng);
}
  /**
   * 获取点集合
   */
  public getPoints(): PIXI.Point[] {
    const points: PIXI.Point[] = [];
    this.pointsGraphics.forEach(item => {
      points.push(item.position);
    });
    return points;
  }
  /**
   * 设置名称显示
   * @param value true/false 显示/隐藏
   * @param mode BasicInformation = 0 基本信息
   *             Assignment想定作业 = 1 想定作业
   */
  public setNameVisible(value: boolean, mode: GameMode) {
    if (this.assetData.GameMode === mode) {
      this.text.visible = value;
    }
  }
  public refresh() {
    this.text.text = this.assetData.Name
      + '\r\n'
      + this.assetData.PropertyInfos.find(item => item.PropertyName === '名称/编号')?.PropertyValue;
    // 填充多边形
    const color: number = this.assetData.Color.substring(0, 7).replace('#', '0x');
    const angle: number = parseInt(this.assetData.Color.substring(7), 16) / 255;
    this.polygonGraphics.clear();
    this.polygonGraphics.beginFill(color, angle);
    this.polygonGraphics.drawPolygon(this.getPoints());
    this.polygonGraphics.endFill();
  }
}
/**
 * 汽车放置区域
 */
export class PutCarArea extends PIXI.Container {
  public polygonGraphics: PIXI.Graphics = new PIXI.Graphics();
  constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
    super();
    this.name = this.assetData.Id;
    this.x = this.assetData.Point.x;
    this.y = this.assetData.Point.y;
    this.workingArea.backgroundImage.addChild(this);
    this.sortableChildren = true;

    // 填充多边形

    const color: number = this.assetData.Color.substring(0, 7).replace('#', '0x');
    const angle: number = parseInt(this.assetData.Color.substring(7), 16) / 255;
    this.polygonGraphics.beginFill(color, angle);
    this.polygonGraphics.drawPolygon(this.assetData.MultiPoint);
    this.polygonGraphics.endFill();
    this.addChild(this.polygonGraphics);
    // 添加选中事件
    this.polygonGraphics.interactive = true;
    this.polygonGraphics
      .on('pointerdown', (event) => {
        if (this.workingArea.getPaintMode() === PaintMode.Car) {
          this.workingArea.selectCar.Point =
            new PIXI.Point(this.workingArea.previewSinglePointIcon.x, this.workingArea.previewSinglePointIcon.y);
          this.workingArea.selectCar.Angle = this.assetData.Direction;
          const car = new SinglePointIcon(this.workingArea.selectCar, this.workingArea);
          this.workingArea.setPaintMode(PaintMode.endPaint);
        }
      })
      .on('pointerup', (event) => {

      })
      .on('pointerupoutside', (event) => {

      })
      .on('pointerover', (event) => {
        this.workingArea.previewSinglePointIcon.filters = null;
        this.workingArea.previewSinglePointIcon.zIndex = this.zIndex + 1;
        // 设置车辆方向
        this.workingArea.previewSinglePointIcon.angle = this.assetData.Direction;
        console.log(this.assetData.Name);
      })
      .on('pointerout', (event) => {
        this.workingArea.previewSinglePointIcon.filters = [
          new OldFilmFilter()
        ];
      });
  }
}
/**
 * 管线
 */
export class Pipeline extends PIXI.Container {
  public line: PIXI.Graphics = new PIXI.Graphics();
  constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
    super();
    this.name = this.assetData.Id;
    this.x = this.assetData.Point.x;
    this.y = this.assetData.Point.y;
    this.workingArea.backgroundImage.addChild(this);
    this.addChild(this.line);
    // 画线图标
    this.refresh();
    this.interactive = true;
    this.on('mousedown', event => {
      console.log(this.assetData);
      event.stopPropagation();
      this.workingArea.selection.selectOne(this);
    });
  }
  /**
   * 刷新
   */
  public refresh() {
    this.line.clear();
    this.line.lineStyle(5, 0x0000ff, 1);
    for (let i = 0, count = this.assetData.MultiPoint.length; i < count; i++) {
      const pointA = this.assetData.MultiPoint[i];
      if (i === 0) {
        this.line.moveTo(pointA.x, pointA.y);
      } else {
        this.line.lineTo(pointA.x, pointA.y);
        if (i === count - 1) {
          const pointB = this.assetData.MultiPoint[i - 1];
          const angle = Math.atan2((pointA.y - pointB.y), (pointA.x - pointB.x)) * (180 / Math.PI) + 90;
          this.line.beginFill(0x0000ff);
          this.line.drawStar(pointA.x, pointA.y, 3, 5, 0, (Math.PI / 180 * angle));
        } else {
          this.line.drawCircle(pointA.x, pointA.y, 5);
        }
      }
    }
    this.line.endFill();
    // if (this.assetData.MultiPoint.length >= 2) {
    //   const pointA = this.assetData.MultiPoint[this.assetData.MultiPoint.length - 2];
    //   const pointB = this.assetData.MultiPoint[this.assetData.MultiPoint.length - 1];
    //   const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI) + 90;
    //   this.beginFill(0x0000ff);
    //   this.drawStar(pointB.x, pointB.y, 3, 5, 0, (Math.PI / 180 * angle));
    //   this.endFill();
    // }
  }
}
/**
 * 箭头
 * 创建一个只有2个点组成的箭头
 */
export class Arrows extends PIXI.Container {
  public line: PIXI.Graphics = new PIXI.Graphics();
  public ready = false;
  constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
    super();
    this.workingArea.backgroundImage.addChild(this);
    this.name = this.assetData.Id;
    this.addChild(this.line);
    this.refresh();
    this.interactive = true;
    this.on('mousedown', event => {
      if (!this.ready) { return; }
      event.stopPropagation();
      this.workingArea.selection.selectOne(this);
    });
  }
  /**
   * 刷新
   */
  public refresh() {
    this.line.clear();
    this.line.lineStyle(5, 0xff0000, 1);
    this.line.moveTo(this.assetData.pointA.x, this.assetData.pointA.y);
    this.line.lineTo(this.assetData.pointB.x, this.assetData.pointB.y);

    const angle = Math.atan2((this.assetData.pointB.y - this.assetData.pointA.y), (this.assetData.pointB.x - this.assetData.pointA.x))
      * (180 / Math.PI) + 90;

    this.line.beginFill(0xff0000);
    console.log(Math.PI / 180 / 1.6);
    this.line.drawStar(this.assetData.pointB.x, this.assetData.pointB.y, 3, 10, 0, (Math.PI / 180 * angle));
    this.line.endFill();
  }
}



/**
 * 选择器
 */
export class Selection {
  constructor(private workingArea: WorkingAreaComponent) {}
  public objects: any[] = [];
  public isMultiselection = false;
  /**
   * 返回选择器中是否包含对象
   * @param obj 对象
   */
  public contains(obj: any): boolean {
    return this.objects.includes(obj);
  }
  /**
   * 选定对象
   * @param obj 对象
   */
  public select(obj: any) {
    if (!this.contains(obj)) {
      this.workingArea.emit('select', obj);
      this.objects.push(obj);
    }
  }
  /**
   * 取消选定对象
   * @param obj 对象
   */
  public deselect(obj: any) {
    if (this.contains(obj)) {
      this.workingArea.emit('deselect', obj);
      const idx = this.objects.findIndex(x => x === obj);
      this.objects.splice(idx, 1);
    }
  }
  /**
   * 选定或取消选定对象
   * @param obj 对象
   */
  public selectOrDeselect(obj: any) {
    if (this.contains(obj)) {
      this.deselect(obj);
    } else {
      this.select(obj);
    }
  }
  /**
   * 取消选定所有已选定对象
   */
  public deselectAll() {
    this.objects.forEach(item => {
      this.workingArea.emit('deselect', item);
    });
    this.objects.splice(0, this.objects.length);
  }
  /**
   * 取消选定所有对象后选定一个对象
   * @param obj 对象
   */
  public selectOne(obj: any) {
    if (this.isMultiselection) {
      this.selectOrDeselect(obj);
    } else {
      this.deselectAll();
      this.select(obj);
    }
  }
  /**
   * 选定对象集合中所有对象
   * @param objects 对象集合
   */
  public selectAll(objects: any[]) {
    this.objects.forEach(item => {
      this.select(item);
    });
  }
}

/**
 * 属性
 */
export class PropertyInfo {
  constructor(instanceData: any) {
    this.Tag = instanceData.tag;
    this.Order = instanceData.order;
    this.Enabled = instanceData.enabled;
    this.Visible = instanceData.visible;
    this.Required = instanceData.required;
    this.RuleName = instanceData.ruleName;
    this.RuleValue = instanceData.ruleValue;
    this.PhysicalUnit = instanceData.physicalUnit;
    this.PropertyName = instanceData.propertyName;
    this.PropertyType = instanceData.propertyType;
    this.PropertyValue = instanceData.propertyValue;
  }
  /**
   * 标记位,用于扩展
   */
  public Tag: string;
  /**
   * 属性排序
   */
  public Order: number;
  /**
   * 是否启用
   */
  public Enabled: boolean;
  /**
   * 是否可见
   */
  public Visible: boolean;
  /**
   * 必填
   */
  public Required: boolean;
  /**
   * 验证规则名称
   */
  public RuleName: string;
  /**
   * 验证规则值
   */
  public RuleValue: string;
  /**
   * 物理单位
   */
  public PhysicalUnit: string;
  /**
   * 属性名称
   */
  public PropertyName: string;
  /**
   * 属性类型
   */
  public PropertyType: number;
  /**
   * 属性值
   */
  public PropertyValue: string;
}
/**
 * 车辆类型
 */
export enum Type {
  水源 = 0,
  举高喷射消防车 = 1,
  泡沫消防车 = 2,
  水罐消防车 = 3,
  压缩空气泡沫消防车 = 4
}
